# 肝脏分割项目

1. 下载并解压 lits 数据集 [地址](https://pan.baidu.com/s/1_M71OfQRl7-jZsZs_sHVOA?pwd=s61k)，每一个文件都有一个对应的 . 开头的隐藏文件，去除文件夹中所有 . 开头的隐藏文件 (这块可以py脚本处理，但是删东西的脚本用错了就很麻烦，手动删除参考[网上教程](https://jingyan.baidu.com/article/4f7d5712dad0135b201927e7.html)。显示隐藏文件之后删掉 . 开头的文件就行)

2. (推荐，可选) 创建虚拟环境，没有 conda 按官网[教程](https://docs.conda.io/en/latest/miniconda.html#latest-miniconda-installer-links)下载[安装包](https://repo.anaconda.com/miniconda/Miniconda3-latest-Windows-x86_64.exe)安装

```shell
conda create -n liver python=3.9 -y
conda activate liver
```

3. 安装项目依赖

Paddle 需要单独安装，cpu 版本不需要装驱动，直接

```shell
pip install paddlepaddle
```

或者按照[官网教程](https://paddlepaddle.org.cn/)安装 GPU 版本。GPU 版本训练速度更快

安装完成后用下面一行代码确认安装成功

```shell
python -c "import paddle; paddle.utils.run_check()"
```

应该看到类似以下输出

```shell
Running verify PaddlePaddle program ...
W0620 15:33:36.613406   144 gpu_context.cc:278] Please NOTE: device: 0, GPU Compute Capability: 6.1, Driver API Version: 11.7, Runtime API Version: 11.2
W0620 15:33:36.631392   144 gpu_context.cc:306] device: 0, cuDNN Version: 8.2.
PaddlePaddle works well on 1 GPU.
PaddlePaddle works well on 1 GPUs.
PaddlePaddle is installed successfully! Let's start deep learning with PaddlePaddle now.
```

<details>
    <summary>可能的bug: Protobuf 相关问题</summary>
    
如果在运行paddle安装检查时看到类似以下报错，里面有 protobuf package的事

```shell
Traceback (most recent call last):
File "<string>", line 1, in <module>
File "/home/lin/miniconda3/envs/liver/lib/python3.9/site-packages/paddle/__init__.py", line 25, in <module>
    from .framework import monkey_patch_variable
File "/home/lin/miniconda3/envs/liver/lib/python3.9/site-packages/paddle/framework/__init__.py", line 17, in <module>
    from . import random  # noqa: F401
File "/home/lin/miniconda3/envs/liver/lib/python3.9/site-packages/paddle/framework/random.py", line 16, in <module>
    import paddle.fluid as fluid
File "/home/lin/miniconda3/envs/liver/lib/python3.9/site-packages/paddle/fluid/__init__.py", line 36, in <module>
    from . import framework
File "/home/lin/miniconda3/envs/liver/lib/python3.9/site-packages/paddle/fluid/framework.py", line 35, in <module>
    from .proto import framework_pb2
File "/home/lin/miniconda3/envs/liver/lib/python3.9/site-packages/paddle/fluid/proto/framework_pb2.py", line 33, in <module>
    _descriptor.EnumValueDescriptor(
File "/home/lin/miniconda3/envs/liver/lib/python3.9/site-packages/google/protobuf/descriptor.py", line 755, in __new__
    _message.Message._CheckCalledFromGeneratedFile()
TypeError: Descriptors cannot not be created directly.
If this call came from a _pb2.py file, your generated code is out of date and must be regenerated with protoc >= 3.19.0.
If you cannot immediately regenerate your protos, some other possible workarounds are:
1. Downgrade the protobuf package to 3.20.x or lower.
2. Set PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION=python (but this will use pure-Python parsing and will be much slower).

More information: https://developers.google.com/protocol-buffers/docs/news/2022-05-06#python-updates
```

可以通过下面这行代码尝试解决

```shell
pip install protobuf==3.20.1
```

</details>

安装完 paddle 后安装其他依赖

```shell
pip install -r requirements.txt
```

4. 在[config.py](./config.py)中修改图像，标签 nii 文件路径和图像标签转 2d png 图片保存路径

5. 用以下脚本将 nii 图像转成 png

```shell
python to_slice.py
```

6. 生成训练，验证和测试集划分文件，以及 labels.txt

```shell
python gen_list.py
```

7. 训练网络

```shell
python train.py
```

8. 训练过程中可以开启 visualdl 实时查看训练状况。执行下面一行后浏览器访问[http://localhost:8040/](http://localhost:8040/)

```shell
visualdl --logdir output
```

注意: 每次删掉任何一个.log文件之后需要重启visualdl

9. 修改[predict.py](./predict.py)中的输入图像和模型文件路径，用下面一行代码跑推理

```shell
python predict.py
```
