import paddlex as pdx
import numpy as np
import nibabel as nib
from tqdm import tqdm
import cc3d

from .util import load_nii, wwwc

### debug
import matplotlib.pyplot as plt
import matplotlib

###


def predict(img, model_path, bs=4, lcc=True):
    model = pdx.load_model(model_path)
    
    if isinstance(img, str):
        img = load_nii(img)
        img = wwwc(img)
    
    pred = np.zeros_like(img)

    batch = []
    for slice_idx in tqdm(range(img.shape[0])):
        batch.append(img[slice_idx, :, :])
        if len(batch) == bs or slice_idx == img.shape[0] - 1:
            res = model.predict(batch)
            res = [r["label_map"] for r in res]
            for res_idx, pred_idx in enumerate(range(slice_idx - len(batch) + 1, slice_idx + 1)):
                pred[pred_idx, :, :] = res[res_idx]
            batch = []
    if lcc:
        cc = cc3d.connected_components(pred, connectivity=6)
        pred_id, pred_count = np.unique(cc, return_counts=True)
        pred_id = pred_id[1:]
        pred_count = pred_count[1:]
        lcc_id = pred_id[np.argmax(pred_count)]

        cc[cc != lcc_id] = 0
        cc[cc == lcc_id] = 1
        pred = cc
    return pred


if __name__ == "__main__":
    res = predict(
        "/home/lin/Desktop/data/liver/Task03_Liver/imagesTr/liver_0.nii.gz",
        "output/2022-06-20T17-11-29/best_model",
    )

    res = np.transpose(res, [2, 1, 0])
    res = res[::-1, ::-1, :]
    pred = nib.Nifti1Image(res, np.eye(4))
    nib.save(pred, "pred.nii.gz")
