import os
import os.path as osp

import cv2
from tqdm import tqdm

from util import load_nii, wwwc
from config import img_slice_path, lab_slice_path, img_path, lab_path

### debug
import matplotlib.pyplot as plt
import matplotlib

matplotlib.use("TkAgg")
###

os.makedirs(img_slice_path, exist_ok=True)
os.makedirs(lab_slice_path, exist_ok=True)

img_paths = [osp.join(img_path, p) for p in os.listdir(img_path)]
lab_paths = [osp.join(lab_path, p) for p in os.listdir(lab_path)]

img_paths.sort()
lab_paths.sort()

slice_count = 0

for img_path, lab_path in tqdm(zip(img_paths, lab_paths), total=len(img_paths)):
    img = load_nii(img_path)
    lab = load_nii(lab_path)

    img = wwwc(img)
    lab[lab == 2] = 1

    img = img.astype("uint8")
    lab = lab.astype("uint8")

    img_name = osp.basename(img_path).split(".")[0]
    lab_name = osp.basename(lab_path).split(".")[0]

    # print(img_path, lab_path)
    # print(img.shape, lab.shape)
    # print(lab.min(), lab.max())
    # print(img.min(), img.max())
    # print("======")

    for slice_idx in range(img.shape[0]):
        img_slice = img[slice_idx, :, :]
        lab_slice = lab[slice_idx, :, :]

        if lab_slice.sum() > 10: # 只保留有超过10px前景的，完全没有肝的片层对训练作用不大
            # plt.imshow(img_slice)
            # plt.show()

            # plt.imshow(lab_slice)
            # plt.show()

            cv2.imwrite(osp.join(img_slice_path, f"{img_name}-{slice_idx}.png"), img_slice)
            cv2.imwrite(osp.join(lab_slice_path, f"{lab_name}-{slice_idx}.png"), lab_slice)
            slice_count += 1

print(f"Generated {slice_count} slices.")