import os.path as osp
import time

import paddlex as pdx
from paddlex import transforms as T

from config import img_slice_path

data_dir = osp.dirname(osp.dirname(img_slice_path))

# print(data_dir)
# input("here")

train_transforms = T.Compose(
    [
        # T.Resize(target_size=512),
        T.RandomHorizontalFlip(),
        T.Normalize(mean=[0.5, 0.5, 0.5], std=[0.5, 0.5, 0.5]),
    ]
)

eval_transforms = T.Compose(
    [
        # T.Resize(target_size=512),
        T.Normalize(mean=[0.5, 0.5, 0.5], std=[0.5, 0.5, 0.5]),
    ]
)

train_dataset = pdx.datasets.SegDataset(
    data_dir=data_dir,
    file_list=osp.join(data_dir, "train_list.txt"),
    # file_list=osp.join(data_dir, "train_small.txt"),
    label_list=osp.join(data_dir, "labels.txt"),
    transforms=train_transforms,
    shuffle=True,
    num_workers=4,
)

eval_dataset = pdx.datasets.SegDataset(
    data_dir=data_dir,
    file_list=osp.join(data_dir, "val_list.txt"),
    # file_list=osp.join(data_dir, "val_small.txt"),
    label_list=osp.join(data_dir, "labels.txt"),
    transforms=eval_transforms,
    num_workers=4,
)

num_classes = len(train_dataset.labels)
model = pdx.seg.BiSeNetV2(num_classes=num_classes)
# model = pdx.seg.UNet(num_classes=num_classes)
model = pdx.seg.DeepLabV3P(num_classes=num_classes, backbone='ResNet50_vd')
model = pdx.seg.DeepLabV3P(num_classes=num_classes, backbone='ResNet101_vd')
model = pdx.seg.HRNet(num_classes=num_classes)
model = pdx.seg.FastSCNN(num_classes=num_classes, use_mixed_loss=True)

model.train(
    num_epochs=10,
    train_dataset=train_dataset,
    train_batch_size=4, # 13g
    eval_dataset=eval_dataset,
    learning_rate=0.01,
    log_interval_steps=1,
    save_dir=f"output/{time.strftime('%Y-%m-%dT%H-%M-%S', time.localtime())}",
    pretrain_weights=None,
)
